function IVi = TPS(K,T,IV,Ki,Ti,lambda,detail)

% Calculates thin plate splines
% INPUTS
%  K = vector size NK for strikes
%  T = vector size NT for maturities
%  IV = matrix size NK x NT for vol (rows are strikes)
%  Ki = fine-mesh vector of strikes where TPS is calculated 
%  Ti = fine-mesh vector of maturities where TPS is calculated 
%  lambda = smoothing parameter (0=no smoothing)
%  detail = detailed output for intermediate calcs (Y/N)
% OUTPUT
%  IVi = smoothed volatiltiy surface

% By Fabrice Douglas Rouah, FRouah.com and Volopta.com

% Arrange the implied volatilities into a vector
z = IV(:);

% Arrange the strikes and maturities into vectors
[Y X] = meshgrid(T,K);
x = X(:)';
y = Y(:)';

% The fine increment vectors on which the TPS points are obtained
xi = Ki;
yi = Ti;

% Obtain the vectors "a" and "b"
[a b A Q1 Q2 R1 R2] = CalculateAandB(x,y,z,lambda,detail);

% Calculate the thin plate interpolated values
for i=1:length(xi)
    for j=1:length(yi)
        IVi(i,j) = TPSpoint(xi(i),yi(j),x,y,a,b);
    end
end

